/**
 * Copyright 2016-2017 Impinj, Inc & IDENTIX - Sinergy
 * @Author Identix
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "ipj_util.h"
#include "iri.h"
#include <getopt.h>

// danger
#include <signal.h>
#include <sys/types.h>
#include <unistd.h>

/* Allocate memory for iri device */
static ipj_iri_device iri_device = { 0 };

/**
 * Main aplication entrypoint
 * @param argc - 
 * @param argv - 
 * @note This aplication expect some parameters
 * 
 */

/* When a SIGUSR1 signal arrives, set this variable. */

bool running = false;
bool shouldHaveRssi = false;
bool inputGiven = false;
uint16_t intputEpcArray[6] = {0,};
uint16_t outputEpcArray[6] = {0,};
bool outputGiven = false;
bool passwordGIven = false;
int desiredValues = E_IPJ_TAG_FLAG_BIT_EPC;

char port[20], inputEpc[30], outputEpc[30];
int power_tx = 23, session = 0, mode = 0,  write_timeout = 50;
uint32_t password = 0x0;

void synch_signal (int sig)
{
    running = false;
}

#define INPUT   0
#define OUTPUT  1
void parseTo(int val)
{
    int len = 0;
    char helperToHex[4] = {0,};

    if (val == INPUT) {
        len = strlen(inputEpc);
   for (uint8_t i = 0; i < len; i = i+4) {
   memcpy(helperToHex, &inputEpc[i], 4);
   intputEpcArray[i / 4] = strtol(helperToHex, NULL, 16);
   }
       
    }
    else if (val == OUTPUT) {
        len = strlen(outputEpc);
   for (uint8_t i = 0; i < len; i = i+4) {
   memcpy(helperToHex, &outputEpc[i], 4);
   outputEpcArray[i / 4] = strtol(helperToHex, NULL, 16);
   }
    }
}

int main(int argc, char **argv)
{
    // Danger zone for killing process and deallocate properly 
    signal(SIGINT, synch_signal); 
    signal(SIGTERM, synch_signal);
    //signal(KILL, synch_signal); 
    

    static struct option param_options[] = 
    {
        { "port",    required_argument,   0, 'p' },
        { "txpower", required_argument,   0, 't' },
        { "mode",    required_argument,   0, 'm' },
        { "session", required_argument,   0, 's' },
        { "rssi",    no_argument,         0, 'r' },
        { "ximout",    required_argument,   0, 'x' },
        { "input",   required_argument,   0, 'i' },
        { "ouput",   required_argument,   0, 'o' },
        { "watchword",required_argument,   0, 'w' },
        { 0,          0,                  0, 0 }
    };

    // arguments parser
    int option_index = 0;
    int c;
    ipj_error error = E_IPJ_ERROR_SUCCESS;
    unsigned int i;
    ipj_key_value key_value[16];
    uint32_t key_value_count = 0;
    ipj_key_list key_list;
    uint32_t key_list_count = 0;

    memset(port,      0, 20);
    memset(inputEpc,  0, 30);
    memset(outputEpc, 0, 30);

    while(true)
    {
        c = getopt_long(argc, argv, ":p:t:m:s:r:i:o:w:x", param_options, &option_index);
        if ( c == -1)
            break;
        
        switch (c)
        {
        case 'p':
            memcpy(port, optarg, strlen(optarg));
            break;
        case 'm':
            mode = atoi(optarg);
            break;
        case 's':
            session = atoi(optarg);
            break;
        case 't':
            power_tx = atoi(optarg);
            break;
        case 'r':
            shouldHaveRssi = true;
            break;
        case 'i':
            memcpy(inputEpc, optarg, strlen(optarg));
            parseTo(INPUT);
            inputGiven = true;
            break;
        case 'o':
            memcpy(outputEpc, optarg, strlen(optarg));
            outputGiven = true;
            parseTo(OUTPUT);
            break;
        case 'w':
            password = strtol(optarg, NULL, 16);
            passwordGIven = true;
            break;
        case 'x':
            write_timeout = atoi(optarg);
        break;
        default:
            break;
        }
    }

    // printf("Paramater port wherre passed with value %s\n %d \n %d \n %d\n", port, mode, session, power_tx);
    fflush(stdout);
     /* Common example setup */
    error = ipj_util_setup(&iri_device, port);
    IPJ_UTIL_RETURN_ON_ERROR(error, "ipj_util_setup");

    /* Configure transmit power */
    error = ipj_set_value(&iri_device, E_IPJ_KEY_ANTENNA_TX_POWER, power_tx * 100);
    IPJ_UTIL_RETURN_ON_ERROR(error, "ipj_set_value E_IPJ_KEY_ANTENNA_TX_POWER");

    /* session */
    error = ipj_set_value(&iri_device, E_IPJ_KEY_INVENTORY_SESSION, session);
    IPJ_UTIL_RETURN_ON_ERROR(error, "ipj_set_value E_IPJ_KEY_INVENTORY_SESSION");

    /* Configure Search Mode */
    error = ipj_set_value(&iri_device, E_IPJ_KEY_INVENTORY_SEARCH_MODE, mode);
    IPJ_UTIL_RETURN_ON_ERROR(error, "ipj_set_value E_IPJ_KEY_INVENTORY_SEARCH_MODE");

    if (shouldHaveRssi)
        desiredValues |= E_IPJ_TAG_FLAG_BIT_RSSI;

    ipj_set_value(&iri_device, E_IPJ_KEY_REPORT_CONTROL_TAG, desiredValues);

    if (inputGiven && outputGiven) 
    {

        // select the epc to write
        error |= ipj_set(&iri_device, E_IPJ_KEY_SELECT_ENABLE, 0, 0, true);
        error |= ipj_set(&iri_device, E_IPJ_KEY_SELECT_TARGET, 0, 0, E_IPJ_SELECT_TARGET_SL_FLAG);
        error |= ipj_set(&iri_device, E_IPJ_KEY_SELECT_ACTION, 0, 0, E_IPJ_SELECT_ACTION_ASLINVA_DSLINVB);
        error |= ipj_set(&iri_device, E_IPJ_KEY_SELECT_MEM_BANK, 0, 0, E_IPJ_MEM_BANK_EPC);
        error |= ipj_set(&iri_device, E_IPJ_KEY_SELECT_POINTER, 0, 0, 0x20);

        if (passwordGIven) {
            ipj_set_value(&iri_device, E_IPJ_KEY_ACCESS_PASSWORD, password);
        }

        error |= ipj_set(&iri_device, E_IPJ_KEY_SELECT_MASK_LENGTH, 0, 0, 96);

         /* Set EPC to random numbers*/
        for (i = 0; i < 6; i++)
        {
             error |= ipj_set(&iri_device, E_IPJ_KEY_SELECT_MASK_VALUE, 0, i, intputEpcArray[i]);
            //  printf("%4X-", intputEpcArray[i]);
        }
        // printf("\n");

        error = ipj_set_value(&iri_device, E_IPJ_KEY_INVENTORY_SELECT_FLAG, E_IPJ_INVENTORY_SELECT_FLAG_SL);
        IPJ_UTIL_RETURN_ON_ERROR(error, "ipj_set_value E_IPJ_KEY_INVENTORY_SELECT_FLAG");

        /* Enable tag write */
        key_value_count = 4;
        IPJ_CLEAR_STRUCT(key_value);
        IPJ_CLEAR_STRUCT(key_list);

        key_value[0].key = E_IPJ_KEY_TAG_OPERATION_ENABLE;
        key_value[0].value = true;

        key_value[1].key = E_IPJ_KEY_TAG_OPERATION;
        key_value[1].value = E_IPJ_TAG_OPERATION_TYPE_WRITE_EPC;

        key_value[2].key = E_IPJ_KEY_WRITE_EPC_LENGTH_CONTROL;
        key_value[2].value = E_IPJ_WRITE_EPC_LENGTH_CONTROL_AUTO;

        key_value[3].key = E_IPJ_KEY_WRITE_WORD_COUNT;
        key_value[3].value = 6;

        key_list_count = 1;
        key_list.key = E_IPJ_KEY_WRITE_DATA;
        key_list.list_count = 6;

        /* Set EPC to random numbers*/
        for (i = 0; i < 6; i++)
        {
            key_list.list[i] = outputEpcArray[i];
        }

        error = ipj_bulk_set(&iri_device, &key_value[0], key_value_count, &key_list, key_list_count);
        IPJ_UTIL_RETURN_ON_ERROR(error, "ipj_bulk_set");
        fflush(stdout);

         /* Start inventory */
        error = ipj_util_perform_inventory(&iri_device, write_timeout);
        IPJ_UTIL_RETURN_ON_ERROR(error, "ipj_util_perform_inventory");

        /* Set EPC to random numbers*/
        for (i = 0; i < 6; i++)
        {
             error |= ipj_set(&iri_device, E_IPJ_KEY_SELECT_MASK_VALUE, 0, i, outputEpcArray[i]);
            //  printf("%4X-", intputEpcArray[i]);
        }
        error = ipj_set_value(&iri_device, E_IPJ_KEY_INVENTORY_SELECT_FLAG, E_IPJ_INVENTORY_SELECT_FLAG_SL);
        IPJ_UTIL_RETURN_ON_ERROR(error, "ipj_set_value E_IPJ_KEY_INVENTORY_SELECT_FLAG");

         error = ipj_util_perform_inventory(&iri_device, write_timeout);

        ipj_set_value(&iri_device, E_IPJ_KEY_SELECT_ENABLE, false);
        fflush(stdout);
    } 
    else
    {
        /* Start inventory */
        error = ipj_start(&iri_device, E_IPJ_ACTION_INVENTORY);
        IPJ_UTIL_RETURN_ON_ERROR(error, "ipj_start E_IPJ_ACTION_INVENTORY");

        running = true;
        /* Call ipj_receive to process tag reports  */
        while(running)
        {
                /* Start inventory */
            error = ipj_receive(&iri_device);
            if(error != 0) break;
            IPJ_UTIL_RETURN_ON_ERROR(error, "ipj_receive");
        }
        error = ipj_util_perform_inventory(&iri_device, 10);
        IPJ_UTIL_RETURN_ON_ERROR(error, "ipj_receive");
        // error = ipj_stop(&iri_device, E_IPJ_ACTION_INVENTORY);
        // IPJ_UTIL_RETURN_ON_ERROR(error, "ipj_stop");
    }
    
    fflush(stdout);
    /* Common example cleanup */
    error = ipj_util_cleanup(&iri_device);
    IPJ_UTIL_RETURN_ON_ERROR(error, "ipj_util_cleanup");
    fflush(stdout);
    return 0;
}
